@@ -1,4 +1,4 @@
-/* $Id: Pop3Client.c,v 1.9 2002/03/12 23:53:15 bluehal Exp $ */
+/* $Id: Pop3Client.c,v 1.17 2003/03/02 02:17:14 bluehal Exp $ */
 /* Author : Scott Holden ( scotth@thezone.net )
    Modified : Yong-iL Joh ( tolkien@mizi.com )
    Modified : Jorge Garca ( Jorge.Garcia@uv.es )
@@ -10,22 +10,31 @@
  * Last Updated : Tue Nov 13 13:45:23 PST 2001
  *
  */
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
 
 #include "Client.h"
 #include "charutil.h"
+#include "regulo.h"
 
 #ifdef USE_DMALLOC
 #include <dmalloc.h>
 #endif
 
-#define	PCU	(pc->u).pop
+extern int Relax;
+
+#define	PCU	(pc->u).pop_imap
 #define POP_DM(pc, lvl, args...) DM(pc, lvl, "pop3: " args)
 
-#ifdef WITH_GCRYPT
-static FILE *authenticate_md5(Pop3 pc, FILE * fp, char *unused);
-static FILE *authenticate_apop(Pop3 pc, FILE * fp, char *apop_str);
+#ifdef HAVE_GCRYPT_H
+static FILE *authenticate_md5( /*@notnull@ */ Pop3 pc, FILE * fp,
+							  char *unused);
+static FILE *authenticate_apop( /*@notnull@ */ Pop3 pc, FILE * fp,
+							   char *apop_str);
 #endif
-static FILE *authenticate_plaintext(Pop3 pc, FILE * fp, char *unused);
+static FILE *authenticate_plaintext( /*@notnull@ */ Pop3 pc, FILE * fp,
+									char *unused);
 
 static struct authentication_method {
 	const char *name;
@@ -34,7 +43,7 @@ static struct authentication_method {
 	FILE *(*auth_callback) (Pop3 pc, FILE * fp, char *apop_str);
 } auth_methods[] = {
 	{
-#ifdef WITH_GCRYPT
+#ifdef HAVE_GCRYPT_H
 	"cram-md5", authenticate_md5}, {
 	"apop", authenticate_apop}, {
 #endif
@@ -42,6 +51,7 @@ static struct authentication_method {
 	NULL, NULL}
 };
 
+/*@null@*/
 FILE *pop3Login(Pop3 pc)
 {
 	int fd;
@@ -77,6 +87,7 @@ FILE *pop3Login(Pop3 pc)
 		}
 	}
 
+
 	/* try each authentication method in turn. */
 	for (a = auth_methods; a->name != NULL; a++) {
 		/* was it specified or did the user leave it up to us? */
@@ -95,13 +106,13 @@ FILE *pop3Login(Pop3 pc)
 	return NULL;
 }
 
-int pop3CheckMail(Pop3 pc)
+int pop3CheckMail( /*@notnull@ */ Pop3 pc)
 {
 	FILE *f;
 	int read;
 	char buf[BUF_SIZE];
 
-	f = pc->open(pc);
+	f = pop3Login(pc);
 	if (f == NULL)
 		return -1;
 
@@ -151,7 +162,6 @@ int pop3Create(Pop3 pc, const char *str)
 	/* If 'str' line is badly formatted, wmbiff won't display the mailbox. */
 	int i;
 	int matchedchars;
-	struct re_registers regs;
 	/* ([^: ]+) user
 	   ([^@]+) or ([^ ]+) password 
 	   ([^: ]+) server 
@@ -161,45 +171,55 @@ int pop3Create(Pop3 pc, const char *str)
 	   use of '@' in passwords
 	 */
 	const char *regexes[] = {
-		"pop3:([^: ]{1,32}) ([^ ]{1,32}) ([^: ]+)( [0-9]+)? *",
-		"pop3:([^: ]{1,32}):([^@]{1,32})@([^: ]+)(:[0-9]+)? *",
+		"pop3:([^: ]{1,32}):([^@]{0,32})@([A-Za-z][-A-Za-z0-9_.]+)(:[0-9]+)?(  *([CcAaPp][-A-Za-z5 ]*))?$",
+		"pop3:([^: ]{1,32}) ([^ ]{1,32}) ([A-Za-z][-A-Za-z0-9_.]+)( [0-9]+)?(  *([CcAaPp][-A-Za-z5 ]*))?$",
+		//      "pop3:([^: ]{1,32}) ([^ ]{1,32}) ([^: ]+)( [0-9]+)? *",
+		// "pop3:([^: ]{1,32}):([^@]{0,32})@([^: ]+)(:[0-9]+)? *",
 		NULL
 	};
+	struct regulo regulos[] = {
+		{1, PCU.userName, regulo_strcpy},
+		{2, PCU.password, regulo_strcpy},
+		{3, PCU.serverName, regulo_strcpy},
+		{4, &PCU.serverPort, regulo_atoi},
+		{6, PCU.authList, regulo_strcpy_tolower},
+		{0, NULL, NULL}
+	};
+
+	if (Relax) {
+		regexes[0] =
+			"pop3:([^: ]{1,32}):([^@]{0,32})@([^/: ]+)(:[0-9]+)?(  *(.*))?$";
+		regexes[1] =
+			"pop3:([^: ]{1,32}) ([^ ]{1,32}) ([^/: ]+)( [0-9]+)?(  *(.*))?$";
+	}
+
+	/* defaults */
+	PCU.serverPort = 110;
+	PCU.authList[0] = '\0';
 
 	for (matchedchars = 0, i = 0;
 		 regexes[i] != NULL && matchedchars <= 0; i++) {
-		matchedchars = compile_and_match_regex(regexes[i], str, &regs);
+		matchedchars = regulo_match(regexes[i], str, regulos);
 	}
 
 	/* failed to match either regex */
 	if (matchedchars <= 0) {
 		pc->label[0] = '\0';
-		POP_DM(pc, DEBUG_ERROR, "Couldn't parse line %s (%d)\n", str,
-			   matchedchars);
+		POP_DM(pc, DEBUG_ERROR, "Couldn't parse line %s (%d)\n"
+			   "  If this used to work, run wmbiff with the -relax option, and\n "
+			   "  send mail to wmbiff-devel@lists.sourceforge.net with the hostname\n"
+			   "  of your mail server.\n", str, matchedchars);
 		return -1;
 	}
-
-	/* copy matches where they belong */
-	copy_substring(PCU.userName, regs.start[1], regs.end[1], str);
-	copy_substring(PCU.password, regs.start[2], regs.end[2], str);
-	copy_substring(PCU.serverName, regs.start[3], regs.end[3], str);
-	if (regs.start[4] != -1)
-		PCU.serverPort = atoi(str + regs.start[4] + 1);
-	else
-		PCU.serverPort = 110;
-
-	grab_authList(str + regs.end[0], PCU.authList);
+	// grab_authList(str + matchedchars, PCU.authList);
 
 	POP_DM(pc, DEBUG_INFO, "userName= '%s'\n", PCU.userName);
-	POP_DM(pc, DEBUG_INFO, "password= is %d characters long\n",
-		   strlen(PCU.password));
 	POP_DM(pc, DEBUG_INFO, "password is %d chars long\n",
 		   strlen(PCU.password));
 	POP_DM(pc, DEBUG_INFO, "serverName= '%s'\n", PCU.serverName);
 	POP_DM(pc, DEBUG_INFO, "serverPort= '%d'\n", PCU.serverPort);
 	POP_DM(pc, DEBUG_INFO, "authList= '%s'\n", PCU.authList);
 
-	pc->open = pop3Login;
 	pc->checkMail = pop3CheckMail;
 	pc->TotalMsgs = 0;
 	pc->UnreadMsgs = 0;
@@ -209,7 +229,7 @@ int pop3Create(Pop3 pc, const char *str)
 }
 
 
-#ifdef WITH_GCRYPT
+#ifdef HAVE_GCRYPT_H
 static FILE *authenticate_md5(Pop3 pc,
 							  FILE * fp,
 							  char *apop_str __attribute__ ((unused)))
@@ -301,9 +321,11 @@ static FILE *authenticate_apop(Pop3 pc, FILE * fp, char *apop_str)
 		return NULL;
 	}
 }
-#endif							/* WITH_GCRYPT */
+#endif							/* HAVE_GCRYPT_H */
 
-static FILE *authenticate_plaintext(Pop3 pc, FILE * fp, char *apop_str
+/*@null@*/
+static FILE *authenticate_plaintext( /*@notnull@ */ Pop3 pc,
+									FILE * fp, char *apop_str
 									__attribute__ ((unused)))
 {
 	char buf[BUF_SIZE];
